import os
import json
import binascii

from rollbot_crawlab.core import CRAWLAB_ROOT


class Data(object):
    username = ''
    password = ''
    token = ''
    api_address = 'http://localhost:8080/api'

    @property
    def dict(self):
        return {
            'username': self.username,
            'password': self.password,
            'token': self.token,
            'api_address': self.api_address,
        }

    @property
    def json(self):
        return json.dumps(self.dict)


class Config(object):
    json_path = os.path.join(CRAWLAB_ROOT, 'config.json')
    data = Data()

    def __init__(self):
        if os.path.exists(self.json_path):
            self.load()
        else:
            self.save()

    def load(self):
        with open(self.json_path) as f:
            data_str = f.read()
            data = json.loads(data_str)
            self.data.username = data.get('username') or ''
            self.data.password = binascii.a2b_hex(data.get('password')).decode()
            self.data.token = data.get('token') or ''
            self.data.api_address = data.get('api_address') or ''

    def save(self):
        data = Data()
        data.username = self.data.username
        data.password = binascii.b2a_hex(self.data.password.encode()).decode()
        data.token = self.data.token
        data.api_address = self.data.api_address
        with open(self.json_path, 'wb') as f:
            f.write(data.json.encode())


config = Config()



import datetime
import hashlib
import mimetypes
import os
import oss2
from scrapy.utils.python import to_bytes


class OssFileUpload:
    def __init__(self, key_id, access_key, endpoint_url, bucket_name):
        self._auth = oss2.Auth(key_id, access_key)
        self._bucket = oss2.Bucket(self._auth, endpoint_url, bucket_name)

    def exist(self, path):
        res = False
        try:
            res = self._bucket.object_exists(path)
        except:
            pass
        return res

    def upload_object(self, path, buf):
        self._bucket.put_object(key=path, data=buf.getvalue())

    def upload_from_file(self, path, file):
        self._bucket.put_object_from_file(key=path, filename=file)

    def get_url_md5(self, url):
        return hashlib.sha1(to_bytes(url)).hexdigest()

    def get_file_path(self, url, item, default_ext, folder):
        media_guid = self.get_url_md5(url)
        media_ext = os.path.splitext(url)[1]
        if media_ext not in mimetypes.types_map:
            media_ext = ''
            media_type = mimetypes.guess_type(url)[0]
            if media_type:
                media_ext = mimetypes.guess_extension(media_type)
        if '' == media_ext:
            media_ext = default_ext
        prefix = ''
        if item is not None:
            prefix = item.get("domain_id", '') + '/' + self.get_url_md5(item.get("source_url", "")) + "/"
        now = datetime.datetime.now().strftime('%Y%m')
        if folder and '' != folder:
            folder += "/"
        return prefix + folder + now + '/' + '%s%s' % (media_guid, media_ext)


import configparser
import importlib
import os
import scrapy


def get_scrapy_cfg():
    cp = configparser.ConfigParser()
    cp.read('scrapy.cfg')
    return cp


def get_items_fields(items):
    data = []
    for key in [key for key in dir(items) if not key.startswith('__')]:
        cls = getattr(items, key)
        try:
            if isinstance(cls(), scrapy.Item):
                d = {
                    'name': key,
                    'fields': []
                }
                if hasattr(cls, 'fields'):
                    d['fields'] = list(cls.fields.keys())
                data.append(d)
        except:
            pass
    return data


def update_items_field(items_data):
    content = ''
    content += 'import scrapy\n\n'


def get_pipelines(pipelines):
    data = []
    for key in [key for key in dir(pipelines) if not key.startswith('__')]:
        cls = getattr(pipelines, key)
        try:
            if isinstance(cls(), object):
                data.append(key)
        except:
            pass

    return data


def get_spider_filepath(filenames, project_name, name):
    for filename in filenames:
        mod_name = filename.replace('.py', '')
        try:
            spider_file = importlib.import_module(f'{project_name}.spiders.{mod_name}')
        except:
            continue
        for key in dir(spider_file):
            try:
                cls = getattr(spider_file, key)
                instance = cls()
                if isinstance(instance, scrapy.Spider):
                    spider_name = getattr(instance, 'name')
                    if spider_name == name:
                        return os.path.join('', project_name, 'spiders', filename)[1:]
            except:
                pass



import psycopg2
import pymysql

from rollbot_crawlab.constants import DataSourceType
from rollbot_crawlab.utils.config import get_data_source, get_data_source_type, get_collection

conn_cache = {}


def get_conn():
    ds_type = get_data_source_type()
    if conn_cache.get(ds_type) is None:
        if ds_type == DataSourceType.MYSQL:
            # MySQL
            conn_cache[ds_type] = connect_mysql()
        elif ds_type == DataSourceType.POSTGRES:
            # Postgres
            conn_cache[ds_type] = connect_postgres()
    return conn_cache.get(ds_type)


def connect_mysql():
    ds = get_data_source()
    return pymysql.connect(
        host=ds.get('host') or 'localhost',
        port=int(ds.get('port')) if ds.get('port') is not None else 3306,
        user=ds.get('username') or None,
        password=ds.get('password') or None,
        database=ds.get('database') or 'test',
        charset='utf8'
    )


def connect_postgres():
    ds = get_data_source()
    return psycopg2.connect(
        host=ds.get('host') or 'localhost',
        port=ds.get('port') or '5432',
        user=ds.get('username') or None,
        password=ds.get('password') or None,
        database=ds.get('database') or 'test',
    )


def _insert_item(item: dict):
    conn = get_conn()
    columns = item.keys()
    table_name = get_collection()
    if table_name is None:
        raise Exception('table_name is empty')
    sql_str = f'INSERT INTO {table_name}({",".join(columns)}) VALUES ({",".join(["%s" for _ in columns])});'
    cursor = conn.cursor()
    cursor.execute(sql_str, list(item.values()))
    conn.commit()
    cursor.close()


def insert_item_mysql(item: dict):
    _insert_item(item)


def insert_item_postgres(item: dict):
    _insert_item(item)


def insert_item_sqlserver(item: dict):
    raise NotImplementedError('sqlserver is not implemented')


def insert_item(item: dict):
    ds_type = get_data_source_type()
    if ds_type == DataSourceType.MYSQL:
        insert_item_mysql(item)
    elif ds_type == DataSourceType.POSTGRES:
        insert_item_postgres(item)
    elif ds_type == DataSourceType.SQLSERVER:
        insert_item_sqlserver(item)
    else:
        raise NotImplementedError(f'{ds_type} is not implemented')


def _update_item(item: dict, dedup_field: str):
    conn = get_conn()
    table_name = get_collection()
    if table_name is None:
        raise Exception('table_name is empty')
    update_str = ','.join([f'{k}=\'{v}\'' for k, v in item.items()])
    sql_str = f'UPDATE {table_name} SET {update_str} WHERE {dedup_field} = \'{item[dedup_field]}\';'
    cursor = conn.cursor()
    cursor.execute(sql_str)
    conn.commit()
    cursor.close()


def update_item_mysql(item: dict, dedup_field: str):
    _update_item(item, dedup_field)


def update_item_postgres(item: dict, dedup_field: str):
    _update_item(item, dedup_field)


def update_item_sqlserver(item: dict, dedup_field: str):
    raise NotImplementedError('sqlserver is not implemented')


def update_item(item: dict, dedup_field: str):
    ds_type = get_data_source_type()
    if ds_type == DataSourceType.MYSQL:
        update_item_mysql(item, dedup_field)
    elif ds_type == DataSourceType.POSTGRES:
        update_item_postgres(item, dedup_field)
    elif ds_type == DataSourceType.SQLSERVER:
        update_item_sqlserver(item, dedup_field)
    else:
        raise NotImplementedError(f'{ds_type} is not implemented')


def _get_item(key: str, value: str) -> dict:
    conn = get_conn()
    table_name = get_collection()
    if table_name is None:
        raise Exception('table_name is empty')
    sql_str = f'SELECT * FROM {table_name} WHERE {key} = \'{value}\'';
    cursor = conn.cursor()
    cursor.execute(sql_str)
    conn.commit()
    res = cursor.fetchone()
    cursor.close()
    return res


def get_item_mysql(key: str, value: str) -> dict:
    return _get_item(key, value)


def get_item_postgres(key: str, value: str) -> dict:
    return _get_item(key, value)


def get_item_sqlserver(key: str, value: str) -> dict:
    raise NotImplementedError('sqlserver is not implemented')


def get_item(item: dict, dedup_field: str) -> dict:
    ds_type = get_data_source_type()
    key = dedup_field
    value = item[dedup_field]
    if ds_type == DataSourceType.MYSQL:
        return get_item_mysql(key, value)
    elif ds_type == DataSourceType.POSTGRES:
        return get_item_postgres(key, value)
    elif ds_type == DataSourceType.SQLSERVER:
        return get_item_sqlserver(key, value)
    else:
        raise NotImplementedError(f'{ds_type} is not implemented')



import os

home = os.curdir

if 'HOME' in os.environ:
    home = os.environ['HOME']
elif os.name == 'posix':
    home = os.path.expanduser('~/')
elif os.name == 'nt':
    if 'HOMEPATH' in os.environ and 'HOMEDRIVE' in os.environ:
        home = os.environ['HOMEDRIVE'] + os.environ['HOMEPATH']
elif 'HOMEPATH' in os.environ:
    home = os.environ['HOMEPATH']

CRAWLAB_ROOT = os.path.join(home, '.crawlab')
CRAWLAB_TMP = os.path.join(CRAWLAB_ROOT, 'tmp')

if not os.path.exists(CRAWLAB_ROOT):
    os.mkdir(CRAWLAB_ROOT)

if not os.path.exists(CRAWLAB_TMP):
    os.mkdir(CRAWLAB_TMP)



import elasticsearch as elasticsearch

from rollbot_crawlab.utils.config import get_data_source


def get_client() -> elasticsearch.Elasticsearch:
    ds = get_data_source()
    return elasticsearch.Elasticsearch(hosts=[{'host': ds.get('host'), 'port': ds.get('port')}])


def index_item(item):
    ds = get_data_source()
    client = get_client()
    client.index(
        index=ds.get('database'),
        body=item,
    )



import json

import requests
from rollbot_crawlab.core.config import config


class Request(object):
    @staticmethod
    def get_error(res):
        try:
            return json.loads(res.content)
        except Exception as err:
            print(err)
            print(res.content)
            return {'error': 'not json content'}

    @staticmethod
    def get(path, params=None):
        try:
            res = requests.get(
                f'{config.data.api_address}{path}',
                params,
                headers={'Authorization': config.data.token},
            )
        except requests.exceptions.ConnectionError as err:
            print(f'error: {err}')
            return {'error': err}
        if res.status_code != 200:
            return Request.get_error(res)
        return json.loads(res.content)

    @staticmethod
    def post(path, data=None):
        try:
            res = requests.post(
                f'{config.data.api_address}{path}',
                json=data,
                headers={'Authorization': config.data.token},
            )
        except requests.exceptions.ConnectionError as err:
            print(f'error: {err}')
            return {'error': err}
        if res.status_code != 200:
            return Request.get_error(res)
        return json.loads(res.content)

    @staticmethod
    def put(path, data=None):
        try:
            res = requests.put(
                f'{config.data.api_address}{path}',
                json=data,
                headers={'Authorization': config.data.token},
            )
        except requests.exceptions.ConnectionError as err:
            print(f'error: {err}')
            return {'error': err}
        if res.status_code != 200:
            return Request.get_error(res)
        return json.loads(res.content)

    @staticmethod
    def delete(path=None):
        try:
            res = requests.delete(
                f'{config.data.api_address}{path}',
                headers={'Authorization': config.data.token},
            )
        except requests.exceptions.ConnectionError as err:
            print(f'error: {err}')
            return {'error': err}
        if res.status_code != 200:
            return Request.get_error(res)
        return json.loads(res.content)

    @staticmethod
    def upload(path=None, file=None, data=None):
        url = f'{config.data.api_address}{path}'
        try:
            res = requests.post(
                url,
                headers={'Authorization': config.data.token},
                data=data,
                files={
                    'file': open(file, 'rb')
                }
            )
        except requests.exceptions.ConnectionError as err:
            return {'error': err}
        if res.status_code != 200:
            return Request.get_error(res)
        return json.loads(res.content)



import datetime
import re
from newspaper import Article
import html
import hashlib
import scrapy


class ArticleItem(scrapy.Item):
    task_id = scrapy.Field()
    domain_url = scrapy.Field()
    domain_id = scrapy.Field()
    domain_name = scrapy.Field()
    source_url = scrapy.Field()
    source_id = scrapy.Field()
    source_name = scrapy.Field()
    block_id = scrapy.Field()
    title = scrapy.Field()
    publish_date = scrapy.Field()
    url = scrapy.Field()
    retweeted_url = scrapy.Field()
    url_md5 = scrapy.Field()
    author = scrapy.Field()
    content = scrapy.Field()
    article_html = scrapy.Field()
    images = scrapy.Field()
    oss_images = scrapy.Field()
    movies = scrapy.Field()
    oss_movies = scrapy.Field()
    other_info = scrapy.Field()
    upload_oss = scrapy.Field()
    download_video = scrapy.Field()
    download_image = scrapy.Field()
    created_date = scrapy.Field()
    crawl_comment = scrapy.Field()
    comment_total_count = scrapy.Field()
    err_msg = scrapy.Field()
    content_type = scrapy.Field()
    post_id = scrapy.Field()  # 所应平台的post id


class ArticleCommentItem(scrapy.Item):
    task_id = scrapy.Field()
    domain_url = scrapy.Field()
    domain_id = scrapy.Field()
    domain_name = scrapy.Field()
    post_url = scrapy.Field()  # post url
    post_id = scrapy.Field()  # 所属对应平台的post id
    block_id = scrapy.Field()
    comment_id = scrapy.Field()  # 评论ID
    url = scrapy.Field()  # 评论链接
    url_md5 = scrapy.Field()
    like_count = scrapy.Field()  # 点赞数
    author = scrapy.Field()  # 评论作者
    author_id = scrapy.Field()  # 评论作者ID，取可以定位到用户首页的ID
    content = scrapy.Field()  # 评论文字内容
    article_html = scrapy.Field()
    other_info = scrapy.Field()
    ip_label = scrapy.Field()  # 评论IP
    upload_oss = scrapy.Field()
    publish_date = scrapy.Field()
    created_date = scrapy.Field()


class ArticleAuthorItem(scrapy.Item):
    task_id = scrapy.Field()
    block_id = scrapy.Field()   # 平台
    url = scrapy.Field()   # 主页链接
    url_md5 = scrapy.Field()
    name = scrapy.Field()   # 昵称
    avatar = scrapy.Field()   # 头像
    describe = scrapy.Field()   # 个性签名
    follow_count = scrapy.Field()   # 关注用户数
    followers_count = scrapy.Field()   # 粉丝数
    location = scrapy.Field()   # 地区
    mobile = scrapy.Field()   # 手机号
    sec_uid = scrapy.Field()
    uid = scrapy.Field()   # 用户ID
    unique_id = scrapy.Field()   # 唯一ID
    age = scrapy.Field()   # 年龄
    other_info = scrapy.Field()   # 其它
    ip_label = scrapy.Field()   # IP标签
    upload_oss = scrapy.Field()
    register_date = scrapy.Field()
    created_date = scrapy.Field()


class PaperItem(scrapy.Item):
    task_id = scrapy.Field()
    domain_url = scrapy.Field()
    domain_id = scrapy.Field()
    domain_name = scrapy.Field()
    source_url = scrapy.Field()
    source_id = scrapy.Field()
    source_name = scrapy.Field()
    block_id = scrapy.Field()
    title_cn = scrapy.Field()
    title_en = scrapy.Field()
    paper_id = scrapy.Field()
    cover_url = scrapy.Field()
    oss_cover_url = scrapy.Field()
    publish_date = scrapy.Field()
    url = scrapy.Field()
    url_md5 = scrapy.Field()
    author = scrapy.Field()
    content = scrapy.Field()
    article_html = scrapy.Field()
    lang = scrapy.Field()   # 正文语言
    abstract_cn = scrapy.Field()   # 摘要
    abstract_en = scrapy.Field()
    keywords = scrapy.Field()   # 关键词
    doi = scrapy.Field()   # DOI
    year = scrapy.Field()   # 年份
    quote = scrapy.Field()   # 引用
    publish_source = scrapy.Field()   # 发布来源
    publish_source_url = scrapy.Field()   # 发布来源URL
    subject = scrapy.Field()   # 学科
    pages = scrapy.Field()   # 页数
    word_number = scrapy.Field()   # 字数
    download_url = scrapy.Field()
    other_info = scrapy.Field()
    upload_oss = scrapy.Field()
    created_date = scrapy.Field()
    err_msg = scrapy.Field()


def get_item(response, source, block, parse=True, uploadOss=True, crawlComment=False):
    item = ArticleItem()
    item["domain_url"] = block["domain_url"]
    item["domain_id"] = block["domain_id"]
    item["domain_name"] = block["domain_name"]
    item["block_id"] = block["identification"]
    item["upload_oss"] = uploadOss
    item["crawl_comment"] = crawlComment
    if source:
        item["source_url"] = source["url"]
        item["source_id"] = source["identification"]
        item["source_name"] = source["name"]
        item["upload_oss"] = source.get('upload_oss', False)
        item["crawl_comment"] = source.get('crawl_comment', False)
    if response:
        item["url"] = response.url
        item["url_md5"] = hashlib.md5(response.url.encode("utf-8")).hexdigest()
    if parse:
        content_regex = block.get('content_regex')
        if content_regex and len(content_regex) > 0:
            manual_parse(item, response, content_regex)
        else:
            auto_parse(item, response)
    return item


def get_comment_item(response, post_result, block, uploadOss=True):
    item = ArticleCommentItem()
    item["domain_url"] = block["domain_url"]
    item["domain_id"] = block["domain_id"]
    item["domain_name"] = block["domain_name"]
    item["block_id"] = block["identification"]
    item["upload_oss"] = uploadOss
    if post_result:
        item["post_url"] = post_result["url"]
        item["post_id"] = post_result["post_id"]
        item["upload_oss"] = post_result.get('upload_oss', False)
    if response:
        item["url"] = response.url
        item["url_md5"] = hashlib.md5(response.url.encode("utf-8")).hexdigest()
    return item


def get_article_author_item(response, author, block, uploadOss=True):
    item = ArticleAuthorItem()
    item["block_id"] = block["identification"]
    item["upload_oss"] = uploadOss
    if author:
        item["upload_oss"] = author.get('upload_oss', False)
    if response:
        item["url"] = response.url
        item["url_md5"] = hashlib.md5(response.url.encode("utf-8")).hexdigest()
    return item


def get_paper_item(response, source, block, parse=True, uploadOss=True):
    item = PaperItem()
    item["domain_url"] = block["domain_url"]
    item["domain_id"] = block["domain_id"]
    item["domain_name"] = block["domain_name"]
    item["block_id"] = block["identification"]
    item["upload_oss"] = uploadOss
    if source:
        item["source_url"] = source["url"]
        item["source_id"] = source["identification"]
        item["source_name"] = source["name"]
        item["upload_oss"] = source.get('upload_oss', False)
    if response:
        item["url"] = response.url
        item["url_md5"] = hashlib.md5(response.url.encode("utf-8")).hexdigest()
    if parse:
        content_regex = block.get('content_regex')
        if content_regex and len(content_regex) > 0:
            manual_parse(item, response, content_regex)
        else:
            auto_parse(item, response)
    return item


def __get_content_html(response, base_url, path):
    html_res = __get_content_extract(response, path)
    if html_res and "" != html_res:
        return get_h5_html(response.url, base_url, html_res)
    return None


def __get_content_text(response, path):
    content_res = __get_content_extract(response, 'string({})'.format(path))
    if content_res and "" != content_res:
        return get_h5_content(content_res)
    return None


def __get_content_images(response, path):
    image_path3 = "{}/descendant::img/@data-echo".format(path)
    images_list3 = response.xpath(image_path3).extract()
    images_list3 = list(map(handle_img_set, list(images_list3)))
    images3 = get_img_list(response, images_list3)

    image_path1 = "{}/descendant::img/@src".format(path)
    images_list1 = response.xpath(image_path1).extract()
    images1 = get_img_list(response, images_list1)

    image_path2 = "{}/descendant::img/@srcset".format(path)
    images_list2 = response.xpath(image_path2).extract()
    images_list2 = list(map(handle_img_set, list(images_list2)))
    images2 = get_img_list(response, images_list2)

    image_path4 = "{}/descendant::img/@data-src".format(path)
    images_list4 = response.xpath(image_path4).extract()
    images4 = get_img_list(response, images_list4)

    item_list = list(set(images1 + images2 + images3 + images4))
    return item_list


def __get_content_videos(response, path):
    movie_path = "{}/descendant::video/@src".format(path)
    movie_list = response.xpath(movie_path).extract()
    movie_last = get_video_list(response, movie_list)
    return movie_last


def add_images(response, item, images):
    images = get_img_list(response, images)
    if images and len(images) > 0:
        if 'images' in item and len(item['images']) > 0:
            item['images'] = item['images'] + images
        else:
            item['images'] = images


def add_videos(response, item, videos):
    videos = get_video_list(response, videos)
    if videos and len(videos) > 0:
        if 'movies' in item and len(item['movies']) > 0:
            item['movies'] = item['movies'] + videos
        else:
            item['movies'] = videos


def manual_parse(item, response, content_regex):
    base_index = response.url.find('/', 8)
    if base_index >= 0:
        base_url = response.url[:base_index]
    else:
        base_url = response.url
    title_path = content_regex.get("title_path")
    if title_path:
        item["title"] = __get_content_extract(response, title_path)
    content_path = content_regex.get("content_path")
    if content_path:
        item['article_html'] = __get_content_html(response, base_url, content_path)
        item['content'] = __get_content_text(response, content_path)
        images = __get_content_images(response, content_path)
        item['images'] = images
        item['movies'] = __get_content_videos(response, content_path)

    content_list = content_regex.get("content_list")
    if content_list and len(content_list) > 0:
        for content_item in content_list:
            name = content_item.get('name')
            path = content_item.get('path')
            if name == "article_html":
                item['article_html'] = __get_content_html(response, base_url, path)
            if name == "content":
                item['content'] = __get_content_text(response, path)
            if name == "images":
                images = __get_content_images(response, path)
                if images:
                    add_images(response, item, images)
            if name == "movies":
                movies = __get_content_videos(response, path)
                if movies:
                    add_videos(response, item, movies)
            if name == "image":
                image = __get_content_extract(response, path)
                if image and '' != image:
                    add_images(response, item, [image])
            if name == "movie":
                movie = __get_content_extract(response, path)
                if movie and '' != movie:
                    add_videos(response, item, [movie])
    publish_path = content_regex.get("publish_path")
    date_format_list = content_regex.get("date_format_list")
    if publish_path and date_format_list:
        date_str = __get_content_extract(response, publish_path)
        item['publish_date'] = get_date(date_str, date_format_list)

    author = {}
    author_name_path = content_regex.get("author_name_path")
    if author_name_path:
        author["name"] = __get_content_extract(response, author_name_path)
    author_avatar_path = content_regex.get("author_avatar_path")
    if author_avatar_path:
        avatar_url = __get_content_extract(response, author_avatar_path)
        author["avatar"] = response.urljoin(avatar_url)
    item['author'] = author

    other_info = {}
    other_paths = content_regex.get("other_paths")
    if other_paths and len(other_paths) > 0:
        for other_path in other_paths:
            name = other_path["name"]
            path = other_path["path"]
            other_info[name] = __get_content_extract(response, path)
    item["other_info"] = other_info


def get_date(date_str, date_strip_list):
    date = None
    for date_strip in date_strip_list:
        try:
            if date_strip == "s":
                s = int(date_str)
                date = datetime.datetime.fromtimestamp(s)
            elif date_strip == "ms":
                ms = int(date_str) / 1000.0
                date = datetime.datetime.fromtimestamp(ms)
            else:
                date = datetime.datetime.strptime(date_str, date_strip)
        except:
            pass
        if date:
            break
    return date


def handle_img_set(image):
    if not image and False == isinstance(image, str):
        return image
    index = image.find(" ")
    if index > 0:
        return image[:index]
    return image


def auto_parse(item, response):
    article = Article(response.url, language='zh', keep_article_html=True, fetch_images=False)
    article.set_html(response.text)
    article.parse()
    authors = list(article.authors)
    item['title'] = article.title
    if len(authors) > 0:
        item['author'] = {
            "name": authors
        }
    item['publish_date'] = article.publish_date
    item['article_html'] = html.unescape(article.article_html)
    item['images'] = list(article.images)
    item['movies'] = list(article.movies)
    item['content'] = article.text
    return item


def __get_content_extract(response, path):
    text = response.xpath(path).extract_first()
    if text:
        return text.replace('\xa0', ' ').strip()
    return text


# 获取文字内容，去重回车空格
def get_h5_content(content):
    if not content or content == "":
        return content
    pattern1 = re.compile('( |\t){2,}')
    pattern2 = re.compile('((\r\n|\n)(\s)*){2,}')
    pattern3 = re.compile('<[^>]+/>')
    pattern4 = re.compile('<[^>]+(([\s\S])*?)<\/[^>]+>')
    content = re.sub(pattern1, " ", content)
    content = re.sub(pattern2, "\r\n", content)
    content = re.sub(pattern3, "\r\n", content)
    content = re.sub(pattern4, "\r\n", content)
    return content


# 获取Html 链接为全路径
def get_h5_html(url, base_url, htmlText):
    url_path = None
    url_host = base_url
    nPos = url.rfind('/') + 1
    if nPos > 0:
        url_path = url[:nPos]
    htmlText = re.sub('style="display:none;"', '', htmlText)
    htmlText = re.sub('<style(((?!>).)*)>(((?!</style>)[\s\S])*)</style>', '', htmlText)
    htmlText = re.sub('<script(((?!>).)*)>(((?!</script>)[\s\S])*)</script>', '', htmlText)
    image_list = re.findall(r'<img(((?!>).)*)data-echo="(((?!").)*)"(((?!>).)*)>', htmlText)
    for image_item in image_list:
        url = image_item[2]
        htmlText = re.sub(r'<img(((?!>).)*)data-echo="%s"(((?!>).)*)>' % url, '<img src="%s">' % url, htmlText)
    if url_path:
        pattern1 = re.compile('(href="(?!http|/|./|../|javascript))|(href="./)')
        pattern3 = re.compile('(src="(?!http|/|./|../|javascript))|(src="./)')
        htmlText = re.sub(pattern1, 'href="%s' % url_path, htmlText)
        htmlText = re.sub(pattern3, 'src="%s' % url_path, htmlText)
        nPrePos = url[:(nPos - 1)].rfind('/')
        if nPrePos >= 10:
            url_pre_path = url[:(nPrePos + 1)]
            patternHref = re.compile('href="../')
            patternSrc = re.compile('href="../')
            htmlText = re.sub(patternHref, 'href="%s' % url_pre_path, htmlText)
            htmlText = re.sub(patternSrc, 'src="%s' % url_pre_path, htmlText)

    if url_host:
        if url_host[len(url_host) - 1] != '/':
            url_host += "/"
        pattern2 = re.compile('href="/')
        pattern4 = re.compile('src="/')
        htmlText = re.sub(pattern2, 'href="%s' % url_host, htmlText)
        htmlText = re.sub(pattern4, 'src="%s' % url_host, htmlText)
    return htmlText


# 获取链接列表
def get_img_list(response, images):
    regex = r'^(https|http)(.*)(jpg|png|jpeg)'
    if not images or len(images) <= 0:
        return images
    img_res = []
    for image in images:
        image = response.urljoin(image)
        if image:
            matchObj = re.search(regex, image, re.I)
            if matchObj:
                img_res.append(image)
    return img_res


def get_video_list(response, videos):
    regex = r'^(https|http)(.*)(jpg|png|jpeg)'
    if not videos or len(videos) <= 0:
        return videos
    video_res = []
    for video in videos:
        video = response.urljoin(video)
        if video:
            matchObj = re.search(regex, video, re.I)
            if matchObj is None:
                video_res.append(video)
    return video_res


def get_json_item(json_item, item_regex):
    if item_regex is None or json_item is None:
        return None
    item_regex_list = item_regex.split('.')
    if len(item_regex_list) > 0:
        res_item = json_item
        for item_regex_item in item_regex_list:
            res_item = res_item.get(item_regex_item)
            if res_item is None:
                return None
        return res_item
    return None


def set_json_item(json_obj, item_regex, val):
    if item_regex is None or json_obj is None:
        return
    item_regex_list = item_regex.split('.')
    if len(item_regex_list) > 0:
        res_item_list = []
        json_item = json_obj
        last_regex_item = None
        for item_regex_item in item_regex_list:
            res_item_list.append({
                'id': last_regex_item,
                'item': json_item
            })
            json_item = json_item.get(item_regex_item)
            if json_item is None:
                json_item = {}
            last_regex_item = item_regex_item
        else:
            res_item_list.append({
                'id': last_regex_item,
                'item': str(val)
            })
        res_item_list.reverse()
        last_item = None
        last_key = None
        for res_item_item in res_item_list:
            id = res_item_item.get('id')
            item = res_item_item.get('item')
            if last_key is not None and last_item is not None:
                item[last_key] = last_item
            last_item = item
            last_key = id
            if last_key is None:
                break
        return last_item



import logging
from contextlib import suppress

import scrapy
from itemadapter import ItemAdapter
from rollbot_crawlab.utils.oss_file_upload import OssFileUpload
from scrapy.exceptions import NotConfigured
from scrapy.pipelines.files import FilesPipeline
from scrapy.utils.log import failure_to_exc_info
from twisted.internet import threads, defer

logger = logging.getLogger(__name__)

class AliOssStore(object):
    file_upload = None

    def __init__(self, ali_oss_config):
        OSS_ACCESS_KEY_ID = ali_oss_config.get("OSS_ACCESS_KEY_ID")
        OSS_SECRET_ACCESS_KEY = ali_oss_config.get("OSS_SECRET_ACCESS_KEY")
        OSS_ENDPOINT_URL = ali_oss_config.get("OSS_ENDPOINT_URL")
        OSS_BUCKET_NAME = ali_oss_config.get("OSS_BUCKET_NAME")
        self.file_upload = OssFileUpload(OSS_ACCESS_KEY_ID, OSS_SECRET_ACCESS_KEY, OSS_ENDPOINT_URL, OSS_BUCKET_NAME)

    def stat_file(self, path, info):
        def _onsuccess(obj):
            return obj
        return threads.deferToThread(self.file_upload.exist, path).addCallback(_onsuccess)

    def persist_file(self, path, buf, info, meta=None, headers=None):
        return threads.deferToThread(self._upload_file, path, buf)

    def _upload_file(self, path, buf):
        self.file_upload.upload_object(path, buf)


class OssFilePipeline(FilesPipeline):
    DEFAULT_FILES_URLS_FIELD = "movies"
    DEFAULT_FILES_RESULT_FIELD = "oss_movies"

    def __init__(self, store_uri, ali_oss_config, settings=None):
        self.ali_oss_config = ali_oss_config
        if store_uri[len(store_uri) - 1] != '/':
            store_uri += "/"
        self.store_uri = store_uri
        super(OssFilePipeline, self).__init__(store_uri=self.store_uri, settings=settings)

    def _get_store(self, uri):
        return AliOssStore(
            self.ali_oss_config
        )

    def get_media_requests(self, item, info):

        upload_oss = item.get('upload_oss', False)
        if upload_oss:
            headers = {
                "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 13_2_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.0.3 Mobile/15E148 Safari/604.1"
            }
            if item.get("block_id") == "post.instagram":
                headers["origin"] = "https://www.instagram.com/"
                headers["referer"] = "https://www.instagram.com/"
            return [scrapy.Request(x, headers=headers, meta={'has_base': True}) for x in item.get(self.files_urls_field, [])]
        else:
            item["download_image"] = False
            item["download_video"] = False
            return []

    @classmethod
    def from_settings(cls, settings):
        ali_oss_config = {
            "OSS_ACCESS_KEY_ID": settings['OSS_ACCESS_KEY_ID'],
            "OSS_SECRET_ACCESS_KEY": settings['OSS_SECRET_ACCESS_KEY'],
            "OSS_ENDPOINT_URL": settings['OSS_ENDPOINT_URL'],
            "OSS_BUCKET_NAME": settings['OSS_BUCKET_NAME']
        }
        store_uri = settings['OSS_BUCKET_URL']
        if not ali_oss_config:
            raise NotConfigured('You should config the ali_oss_config to enable this pipeline')
        return cls(store_uri=store_uri, ali_oss_config=ali_oss_config, settings=settings)

    def media_to_download(self, request, info, *, item=None):
        def _onsuccess(result):
            if not result:
                return  # returning None force download
            return {'url': request.url, 'path': path}

        path = self.file_path(request, info=info, item=item)
        dfd = defer.maybeDeferred(self.store.stat_file, path, info)
        dfd.addCallbacks(_onsuccess, lambda _: None)
        dfd.addErrback(
            lambda f:
            logger.error(self.__class__.__name__ + '.store.stat_file',
                         exc_info=failure_to_exc_info(f),
                         extra={'spider': info.spider})
        )
        return dfd

    def map_item_for_oss_url(self, item):
        item["oss_url"] = self.store_uri + item.get("path")
        return item


class ImageFilesPipeline(OssFilePipeline):
    DEFAULT_FILES_URLS_FIELD = "images"
    DEFAULT_FILES_RESULT_FIELD = "oss_images"

    def item_completed(self, results, item, info):
        with suppress(KeyError):
            if results and len(results) > 0:
                res_list = [x for ok, x in results if ok]
                if len(res_list) > 0:
                    res_list = list(map(self.map_item_for_oss_url, res_list))
                    article_html = ItemAdapter(item).get("article_html")
                    if article_html and '' != article_html:
                        for res_item in res_list:
                            request_url = res_item.get('url')
                            oss_url = res_item.get('oss_url')
                            article_html = article_html.replace(request_url, oss_url)
                        ItemAdapter(item)["article_html"] = article_html
                    ItemAdapter(item)[self.files_result_field] = res_list
                    ItemAdapter(item)["download_image"] = True
                else:
                    ItemAdapter(item)["download_image"] = False
        return item

    def file_path(self, request, response=None, info=None, item=None):
        return self.store.file_upload.get_file_path(
            url=request.url, item=item, default_ext=".jpg", folder="image"
        )


class VideoFilesPipeline(OssFilePipeline):
    DEFAULT_FILES_URLS_FIELD = "movies"
    DEFAULT_FILES_RESULT_FIELD = "oss_movies"

    def item_completed(self, results, item, info):
        with suppress(KeyError):
            if results and len(results) > 0:
                res_list = [x for ok, x in results if ok]
                if len(res_list) > 0:
                    res_list = list(map(self.map_item_for_oss_url, res_list))
                    article_html = ItemAdapter(item).get("article_html")
                    if article_html and '' != article_html:
                        for res_item in res_list:
                            request_url = res_item.get('url')
                            oss_url = res_item.get('oss_url')
                            article_html = article_html.replace(request_url, oss_url)
                        ItemAdapter(item)["article_html"] = article_html
                    ItemAdapter(item)[self.files_result_field] = res_list
                    ItemAdapter(item)["download_video"] = True
                else:
                    ItemAdapter(item)["download_video"] = False
        return item

    def file_path(self, request, response=None, info=None, item=None):
        return self.store.file_upload.get_file_path(
            url=request.url, item=item, default_ext=".mp4", folder="video"
        )


class AuthorAvatarFilesPipeline(OssFilePipeline):
    def get_media_requests(self, item, info):
        headers = {
            "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 13_2_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.0.3 Mobile/15E148 Safari/604.1"
        }
        upload_oss = item.get('upload_oss', False)
        request_list = []
        if upload_oss:
            author_avatar = ItemAdapter(item).get('author', {}).get("avatar")
            if author_avatar:
                request_list.append(
                    scrapy.Request(author_avatar, headers=headers, meta={'has_base': True})
                )
        else:
            item["download_image"] = False
            item["download_video"] = False
        return request_list

    def item_completed(self, results, item, info):
        with suppress(KeyError):
            if 'author' in item:
                res_list = [x for ok, x in results if ok]
                if len(res_list) > 0:
                    res_list = list(map(self.map_item_for_oss_url, res_list))
                    article_html = ItemAdapter(item).get("article_html")
                    if article_html and '' != article_html:
                        for res_item in res_list:
                            request_url = res_item.get('url')
                            oss_url = res_item.get('oss_url')
                            article_html = article_html.replace(request_url, oss_url)
                        ItemAdapter(item)["article_html"] = article_html
                    if res_list and len(res_list) > 0:
                        ItemAdapter(item)['author']['avatar'] = res_list[0].get("oss_url")
        return item

    def file_path(self, request, response=None, info=None, item=None):
        return self.store.file_upload.get_file_path(
            url=request.url, item=item, default_ext=".jpg", folder="avatar"
        )



import os

from pymongo import MongoClient

from rollbot_crawlab.utils.config import get_data_source
g_mongo = None
g_db = None

def get_col(collection=None):
    global g_mongo
    global g_db
    if g_mongo is None:
        ds = get_data_source()

        if ds.get('type') is None:
            # default data source
            mongo_host = os.environ.get('CRAWLAB_MONGO_HOST') or 'localhost'
            mongo_port = int(os.environ.get('CRAWLAB_MONGO_PORT') or 27017) or 27017
            mongo_db = os.environ.get('CRAWLAB_MONGO_DB') or 'test'
            mongo_username = os.environ.get('CRAWLAB_MONGO_USERNAME') or ''
            mongo_password = os.environ.get('CRAWLAB_MONGO_PASSWORD') or ''
            mongo_authsource = os.environ.get('CRAWLAB_MONGO_AUTHSOURCE') or 'admin'
            g_mongo = MongoClient(
                host=mongo_host,
                port=mongo_port,
                username=mongo_username,
                password=mongo_password,
                authSource=mongo_authsource,
            )
            g_db = g_mongo.get_database(mongo_db)
        else:
            # specified mongo data source
            mongo = MongoClient(
                host=ds.get('host'),
                port=int(ds.get('port')),
                username=ds.get('username'),
                password=ds.get('password'),
                authSource=ds.get('auth_source') or 'admin',
            )
            g_db = mongo.get_database(ds.get('database'))
    if collection is None:
        collection = os.environ.get('CRAWLAB_COLLECTION') or 'test'
    col = g_db.get_collection(collection)
    return col




import json

from kafka import KafkaProducer

from rollbot_crawlab.utils.config import get_data_source


def get_producer():
    ds = get_data_source()
    if ds.get('username') is not None and len(ds.get('username')) > 0:
        return KafkaProducer(
            sasl_mechanism="PLAIN",
            security_protocol='SASL_PLAINTEXT',
            sasl_plain_username=ds.get('username'),
            sasl_plain_password=ds.get('password'),
            bootstrap_servers=f'{ds.get("host")}:{ds.get("port")}'
        )
    else:
        return KafkaProducer(
            bootstrap_servers=f'{ds.get("host")}:{ds.get("port")}'
        )


def send_msg(item):
    ds = get_data_source()
    producer = get_producer()
    producer.send(ds.get('database'), json.dumps(item).encode('utf-8'))



import datetime

from rollbot_crawlab.db.mongo import get_col


def get_block(identify):
    block_col = get_col("blocks")
    block = block_col.find_one({"identification": identify})
    if block is None:
        raise Exception("获取block信息失败：{}".format(identify))
    return block


def get_source_verify(url, block):
    col = get_col("sources")
    source = col.find_one({"url": url})
    if source is None \
            or 'enabled' not in source \
            or source['enabled'] is False \
            or 'block_id' not in source \
            or 'url' not in source \
            or 'identification' not in source:
        raise Exception("来源已不可用：{}".format(url))
    if source.get("block_id") != block.get("identification"):
        raise Exception("来源和对应block不匹配：{} - {}".format(url, block.get("identification")))
    source["total_count"] = get_source_total(source)
    return source


def get_source_item(url):
    col = get_col("sources")
    source = col.find_one({"url": url})
    return source


def get_source(url):
    block = None
    source = get_source_item(url)
    if source is None \
            or 'enabled' not in source \
            or source['enabled'] is False \
            or 'block_id' not in source \
            or 'url' not in source \
            or 'identification' not in source:
        raise Exception("来源已不可用：{}".format(url))
    block_id = source.get("block_id")
    if block_id:
        block_col = get_col("blocks")
        block = block_col.find_one({"identification": block_id})
    if block is None:
        raise Exception("来源相关block未找到：{} - {}".format(url, block_id))
    source["total_count"] = get_source_total(source)
    return source, block


def update_source_begin(source):
    col = get_col("sources")
    col.update_one({"_id": source["_id"]}, {"$set": {"latest_start_at": datetime.datetime.now()}})


def update_source_error(source, err_msg):
    if source:
        col = get_col("sources")
        col.update_one({"_id": source["_id"]}, {"$set": {"err_msg": err_msg}})
    raise Exception(err_msg)


def get_source_total(source):
    col = get_col()
    where = {"source_id": source["identification"]}
    return col.count(where)


def update_source_latest(source, latest_url):
    col = get_col("sources")
    latest_count = source.get("total_count", 0)
    total_count= get_source_total(source)
    where = {"_id": source["_id"]}

    json_obj = {
        "latest_count": total_count - latest_count,
        "latest_end_at": datetime.datetime.now(),
        "total_count": total_count
    }

    if latest_url:
        json_obj["latest_url"] = latest_url

    update = {
        "$set": json_obj
    }
    col.update_one(where, update)



from rollbot_crawlab.utils import save_item_mongo
from rollbot_crawlab.utils.config import get_task_id
import datetime


class CrawlabMongoPipeline(object):
    def process_item(self, item, spider):
        item_dict = dict(item)
        item_dict['task_id'] = get_task_id()
        item_dict['created_at'] = datetime.datetime.now()
        save_item_mongo(item_dict)
        return item



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: zengjin
#############################################


from setuptools import setup, find_packages

setup(
    name = "rollbot-crawlab-rollong",
    version = "0.1.23",
    keywords = ("pip", "pathtool","timetool", "magetool", "mage"),
    description = "rollbot crawlab",
    long_description = "rollbot crawlab",
    license = "MIT Licence",

    url = "",
    author = "zengjin",
    author_email = "jzeng@rollong.com",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires = []
)


from rollbot_crawlab.constants import DataSourceType
from rollbot_crawlab.utils.config import get_data_source_type
from rollbot_crawlab.utils.data import save_item_mongo, save_item_sql, save_item_kafka, save_item_es


def save_item(item):
    try:
        if get_data_source_type() == DataSourceType.MONGO:
            save_item_mongo(item)
        elif get_data_source_type() == DataSourceType.KAFKA:
            save_item_kafka(item)
        elif get_data_source_type() == DataSourceType.ELASTICSEARCH:
            save_item_es(item)
        else:
            save_item_sql(item)
    except Exception as ex:
        print(ex)
        pass



import json
import os


def get_task_id():
    return os.environ.get('CRAWLAB_TASK_ID')


def get_is_dedup():
    return os.environ.get('CRAWLAB_IS_DEDUP')


def get_dedup_field():
    return os.environ.get('CRAWLAB_DEDUP_FIELD')


def get_dedup_method():
    return os.environ.get('CRAWLAB_DEDUP_METHOD')


def get_collection():
    return os.environ.get('CRAWLAB_COLLECTION')


def get_data_source_type():
    ds = get_data_source()
    return ds.get('type') or 'mongo'


def get_data_source():
    if os.environ.get('CRAWLAB_DATA_SOURCE') is None:
        return {}
    try:
        ds = json.loads(os.environ.get('CRAWLAB_DATA_SOURCE'))
        return ds
    except:
        return {}



class Spider:
    CUSTOMIZED = 'customized'
    CONFIGURABLE = 'configurable'


class DedupMethod:
    OVERWRITE = 'overwrite'
    IGNORE = 'ignore'


class DataSourceType:
    MONGO = 'mongo'
    MYSQL = 'mysql'
    POSTGRES = 'postgres'
    SQLSERVER = 'sqlserver'
    KAFKA = 'kafka'
    ELASTICSEARCH = 'es'



from rollbot_crawlab.constants import DedupMethod
from rollbot_crawlab.db.es import index_item
from rollbot_crawlab.db.kafka import send_msg
from rollbot_crawlab.db.mongo import get_col
from rollbot_crawlab.db.sql import get_item, update_item, insert_item
from rollbot_crawlab.utils.config import get_task_id, get_is_dedup, get_dedup_field, get_dedup_method
import datetime


def save_item_mongo(item):
    col = get_col()

    # 赋值task_id
    item['task_id'] = get_task_id()

    # 是否开启去重
    is_dedup = get_is_dedup()

    if is_dedup == '1':
        # 去重
        dedup_field = get_dedup_field()
        dedup_method = get_dedup_method()
        if dedup_method == DedupMethod.OVERWRITE:
            # 覆盖
            if col.find_one({dedup_field: item[dedup_field]}):
                col.replace_one({dedup_field: item[dedup_field]}, item)
            else:
                col.save(item)
        elif dedup_method == DedupMethod.IGNORE:
            # 忽略
            col.save(item)
        else:
            # 其他
            col.save(item)
    else:
        # 不去重
        col.save(item)


def save_item_sql(item):
    # 是否开启去重
    is_dedup = get_is_dedup()

    # 赋值task_id
    item['task_id'] = get_task_id()

    if is_dedup == '1':
        # 去重
        dedup_field = get_dedup_field()
        dedup_method = get_dedup_method()
        if dedup_method == DedupMethod.OVERWRITE:
            # 覆盖
            if get_item(item, dedup_field):
                update_item(item, dedup_field)
            else:
                insert_item(item)
        elif dedup_method == DedupMethod.IGNORE:
            # 忽略
            insert_item(item)
        else:
            # 其他
            insert_item(item)
    else:
        # 不去重
        insert_item(item)


def save_item_kafka(item):
    item['task_id'] = get_task_id()
    send_msg(item)


def save_item_es(item):
    item['task_id'] = get_task_id()
    index_item(item)


def get_post_item(url):
    col = get_col("results_rollbot")
    post_result = col.find_one({"url": url})
    return post_result


# 获取post的评论数量
def get_comment_total_by_post_item(post_item):
    col = get_col("results_comments")
    where = {"post_id": post_item["_id"]}
    return col.count(where)


def update_post_item_comment_latest_id(post_item, comment_latest_id):
    col = get_col("results_rollbot")
    comment_latest_count = post_item.get("comment_total_count", 0)
    comment_total_count = get_comment_total_by_post_item(post_item)
    where = {"_id": post_item["_id"]}

    json_obj = {
        "comment_latest_count": comment_total_count - comment_latest_count,
        "latest_end_at": datetime.datetime.now(),
        "comment_total_count": comment_total_count
    }

    if comment_latest_id:
        json_obj["comment_latest_id"] = comment_latest_id

    update = {
        "$set": json_obj
    }
    col.update_one(where, update)


def update_post_item_error(post, err_msg):
    if post:
        col = get_col("result_rollbot")
        col.update_one({"_id": post["_id"]}, {"$set": {"err_msg": err_msg}})
    raise Exception(err_msg)